#include "theme_creator.h"

/*
 *  Constructs a Creator as a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 */
Creator::Creator( QWidget* parent, const char* name, WFlags fl ) : KMainWindow( parent, name, fl ){
    if ( !name ){
      setName( "CreatorDlg" );
    }

    tabWidget = new QTabWidget( this, "tabWidget" );
    tabWidget->setEnabled( false );
    tabWidget->setAutoMask( TRUE );
    setCentralWidget(tabWidget);

    textEdit1 = new KTextEdit( this, "textEdit1" );
    textEdit1->setReadOnly( TRUE );
    textEdit1->setEnabled( false );
    textEdit1->setWordWrap(QTextEdit::NoWrap);
    textEdit1->setMinimumSize(200, 100);
    prevpara = 0;
    tabWidget->insertTab( textEdit1, "" );

    preview = new QWidget( tabWidget, "preview" );
    tabWidget->insertTab( preview, "" );

    // actions
    fileNewAction = new QAction( this, "fileNewAction" );
    fileNewAction->setIconSet( SmallIconSet( "filenew" ));
    fileOpenAction = new QAction( this, "fileOpenAction" );
    fileOpenAction->setIconSet( SmallIconSet( "fileopen" ));
    fileSaveAction = new QAction( this, "fileSaveAction" );
    fileSaveAction->setIconSet( SmallIconSet( "filesave" ));
    fileSaveAsAction = new QAction( this, "fileSaveAsAction" );
    fileSaveAsAction->setIconSet( SmallIconSet( "filesaveas" ));
    filePrintAction = new QAction( this, "filePrintAction" );
    filePrintAction->setIconSet( SmallIconSet( "fileprint" ));
    fileExitAction = new QAction( this, "fileExitAction" );
    fileExitAction->setIconSet( SmallIconSet( "exit" ));

    editUndoAction = new QAction( this, "editUndoAction" );
    editUndoAction->setIconSet( SmallIconSet( "undo" ));
    editRedoAction = new QAction( this, "editRedoAction" );
    editRedoAction->setIconSet( SmallIconSet( "redo" ));
    editCutAction = new QAction( this, "editCutAction" );
    editCutAction->setIconSet( SmallIconSet( "editcut" ));
    editCopyAction = new QAction( this, "editCopyAction" );
    editCopyAction->setIconSet( SmallIconSet( "editcopy" ));
    editPasteAction = new QAction( this, "editPasteAction" );
    editPasteAction->setIconSet( SmallIconSet( "editpaste" ));
    editFindAction = new QAction( this, "editFindAction" );
    editFindAction->setIconSet( SmallIconSet( "find" ));

    helpContentsAction = new QAction( this, "helpContentsAction" );
    helpIndexAction = new QAction( this, "helpIndexAction" );
    helpIndexAction->setIconSet( SmallIconSet( "help" ));
    helpAboutAction = new QAction( this, "helpAboutAction" );
    helpAboutAction->setIconSet( SmallIconSet( "KarthemeCreator" ));
    AboutKDE = new QAction( this, "AboutKDE" );
    AboutKDE->setIconSet( SmallIconSet( "about_kde" ));

    Text = new QAction( this, "Text" );
    Text->setIconSet( SmallIconSet( "edit" ));
    Image = new QAction( this, "Image" );
    Image->setIconSet( SmallIconSet( "thumbnail" ));
    Bar = new QAction( this, "Bar" );
    Bar->setIconSet( SmallIconSet( "blend" ));
    Graph = new QAction( this, "Graph" );
    Graph->setIconSet( SmallIconSet( "kchart" ));
    Theme = new QAction( this, "Theme" );
    Theme->setIconSet( SmallIconSet( "misc" ));
    Child = new QAction( this, "Child" );
    Child->setIconSet( SmallIconSet( "view_tree" ));
    Font = new QAction( this, "Font" );
    Font->setIconSet( SmallIconSet( "fonts" ));
    Group = new QAction( this, "Group" );
    Group->setIconSet( SmallIconSet( "attach" ));
    Click = new QAction( this, "Click" );
    Click->setIconSet( SmallIconSet( "button" ));
    Icon = new QAction( this, "Icon" );
    Icon->setIconSet( SmallIconSet( "view_icon" ));
    Run = new QAction( this, "Run" );
    Run->setIconSet( SmallIconSet( "run" ));
    Stop = new QAction( this, "Stop" );
    Stop->setIconSet( SmallIconSet( "stop" ));

    // toolbars
    toolBar = new KToolBar( this, DockTop, "" );
    //toolBar->setFullSize(true);

    fileNewAction->addTo( toolBar );
    fileOpenAction->addTo( toolBar );
    fileSaveAction->addTo( toolBar );
    fileSaveAsAction->addTo( toolBar );
    editUndoAction->addTo( toolBar );
    editRedoAction->addTo( toolBar );

    editBar = new KToolBar( this, DockTop, "" );
    //editBar->setFullSize(true);

    editCopyAction->addTo( editBar );
    editCutAction->addTo( editBar );
    editPasteAction->addTo( editBar );
    editFindAction->addTo( editBar );

    Karamba_Toolbar = new KToolBar( this, DockLeft, "" );
    Karamba_Toolbar->setFullSize(true);

    Text->addTo( Karamba_Toolbar );
    Image->addTo( Karamba_Toolbar );
    Bar->addTo( Karamba_Toolbar );
    Graph->addTo( Karamba_Toolbar );
    Theme->addTo( Karamba_Toolbar );
    Child->addTo( Karamba_Toolbar );
    Font->addTo( Karamba_Toolbar );
    Group->addTo( Karamba_Toolbar );
    Click->addTo( Karamba_Toolbar );
    Icon->addTo( Karamba_Toolbar );
    Run->addTo( Karamba_Toolbar );
    Stop->addTo( Karamba_Toolbar );


    // menubar
    menubar = new QMenuBar( this, "menubar" );

    fileMenu = new QPopupMenu( this );

    fileNewAction->addTo( fileMenu );
    fileOpenAction->addTo( fileMenu );
    fileSaveAction->addTo( fileMenu );
    fileSaveAsAction->addTo( fileMenu );
    Run->addTo( fileMenu );
    Stop->addTo( fileMenu );
    fileMenu->insertSeparator();
    filePrintAction->addTo( fileMenu );
    fileMenu->insertSeparator();
    fileExitAction->addTo( fileMenu );
    menubar->insertItem( "", fileMenu, 0 );
    editMenu = new QPopupMenu( this );

    editUndoAction->addTo( editMenu );
    editRedoAction->addTo( editMenu );
    editMenu->insertSeparator();
    editCutAction->addTo( editMenu );
    editCopyAction->addTo( editMenu );
    editPasteAction->addTo( editMenu );
    editMenu->insertSeparator();
    editFindAction->addTo( editMenu );
    menubar->insertItem( "", editMenu, 1 );

    PopupMenu = new QPopupMenu( this );

    Text->addTo( PopupMenu );
    Image->addTo( PopupMenu );
    Bar->addTo( PopupMenu );
    Graph->addTo( PopupMenu );
    Theme->addTo( PopupMenu );
    Child->addTo( PopupMenu );
    Font->addTo( PopupMenu );
    Group->addTo( PopupMenu );
    Click->addTo( PopupMenu );
    Icon->addTo( PopupMenu );
    menubar->insertItem( "", PopupMenu, 2 );

    helpMenu = new QPopupMenu( this );

    helpContentsAction->addTo( helpMenu );
    helpIndexAction->addTo( helpMenu );
    helpMenu->insertSeparator();
    helpAboutAction->addTo( helpMenu );
    AboutKDE->addTo( helpMenu );
    menubar->insertItem( "", helpMenu, 3 );

    fileSaveAction->setEnabled(false);
    fileSaveAsAction->setEnabled(false);
    filePrintAction->setEnabled(false);
    editCutAction->setEnabled(false);
    editCopyAction->setEnabled(false);
    editPasteAction->setEnabled(false);
    editFindAction->setEnabled(false);
    editUndoAction->setEnabled(false);
    editUndoAction->setEnabled(false);
    editUndoAction->setEnabled(false);
    editRedoAction->setEnabled(false);
    Text->setEnabled(false);
    Image->setEnabled(false);
    Bar->setEnabled(false);
    Graph->setEnabled(false);
    Theme->setEnabled(false);
    Child->setEnabled(false);
    Font->setEnabled(false);
    Group->setEnabled(false);
    Click->setEnabled(false);
    Icon->setEnabled(false);
    Run->setEnabled(false);
    Stop->setEnabled(false);

    languageChange();

    fn = new KTempFile (locateLocal("tmp", "kar"), ".theme", 0600);
    k = 0;
    kp = 0;
    KarPrevList = 0;
    KarRunList = 0;
    status = KMainWindow::statusBar();
    status->setSizeGripEnabled(true);
    status->insertItem(tr2i18n("Line: 0"), 1, 0, true);
    status->setItemFixed(1, 90);
    status->setItemAlignment( 1, int( QLabel::AlignVCenter | QLabel::AlignLeft ) );
    status->insertItem(tr2i18n("Col: 0"), 2, 0, true);
    status->setItemAlignment( 2, int( QLabel::AlignVCenter | QLabel::AlignLeft ) );
    status->setItemFixed(2, 90);

    resize(640, 600);
    // signals and slots connections
    connect(qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
    connect( fileNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    connect( fileSaveAction, SIGNAL( activated() ), this, SLOT( fileSave() ) );
    connect( fileSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
    connect( filePrintAction, SIGNAL( activated() ), this, SLOT( filePrint() ) );
    connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
    connect( editUndoAction, SIGNAL( activated() ), this, SLOT( editUndo() ) );
    connect( editRedoAction, SIGNAL( activated() ), this, SLOT( editRedo() ) );
    connect( editCutAction, SIGNAL( activated() ), this, SLOT( editCut() ) );
    connect( editCopyAction, SIGNAL( activated() ), this, SLOT( editCopy() ) );
    connect( editPasteAction, SIGNAL( activated() ), this, SLOT( editPaste() ) );
    connect( editFindAction, SIGNAL( activated() ), this, SLOT( editFind() ) );
    connect( helpIndexAction, SIGNAL( activated() ), this, SLOT( helpIndex() ) );
    connect( helpContentsAction, SIGNAL( activated() ), this, SLOT( helpContents() ) );
    connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
    connect( AboutKDE, SIGNAL( activated() ), this, SLOT( aboutKDE() ) );
    connect( Run, SIGNAL( activated() ), this, SLOT( RunTheme() ) );
    connect( Stop, SIGNAL( activated() ), this, SLOT( stopKaramba() ) );
    connect( Image, SIGNAL( activated() ), this, SLOT( AddImage() ) );
    connect( Icon, SIGNAL( activated() ), this, SLOT( AddIcon() ) );
    connect( Text, SIGNAL( activated() ), this, SLOT( AddText() ) );
    connect( Bar, SIGNAL( activated() ), this, SLOT( AddBar() ) );
    connect( Graph, SIGNAL( activated() ), this, SLOT( AddGraph() ) );
    connect( Theme, SIGNAL( activated() ), this, SLOT( AddTheme() ) );
    connect( Child, SIGNAL( activated() ), this, SLOT( AddChild() ) );
    connect( Click, SIGNAL( activated() ), this, SLOT( AddClick() ) );
    connect( Group, SIGNAL( activated() ), this, SLOT( AddGroup() ) );
    connect( Font, SIGNAL( activated() ), this, SLOT( AddFont() ) );
    connect( textEdit1, SIGNAL( copyAvailable ( bool ) ), editCopyAction, SLOT( setEnabled( bool ) ) );
    connect( textEdit1, SIGNAL( copyAvailable ( bool ) ), editCutAction, SLOT( setEnabled( bool ) ) );
    connect( textEdit1, SIGNAL( undoAvailable ( bool ) ), editUndoAction, SLOT( setEnabled( bool ) ) );
    connect( textEdit1, SIGNAL( redoAvailable ( bool ) ), editRedoAction, SLOT( setEnabled( bool ) ) );
    connect( textEdit1, SIGNAL( modificationChanged ( bool ) ), this, SLOT( setModif( bool ) ) );
    connect( textEdit1, SIGNAL( modificationChanged ( bool ) ), this, SLOT( parse( ) ) );
    connect( textEdit1, SIGNAL( cursorPositionChanged ( int, int ) ), this, SLOT( upCurPos( int, int ) ) );
    connect( tabWidget, SIGNAL(currentChanged( QWidget* )), this, SLOT( slotPreview(QWidget*) ) );

}

/*
 *  Destroys the object and frees any allocated resources
 */
Creator::~Creator(){
    // no need to delete child widgets, Qt does it all for us
    fn->unlink();
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void Creator::languageChange(){
    setCaption( tr2i18n( "Karamba Theme Creator" ) );
    textEdit1->setText( tr2i18n( "Choose New or Open theme..." ) );
    fileNewAction->setText( tr2i18n( "New" ) );
    fileNewAction->setMenuText( tr2i18n( "&New" ) );
    fileNewAction->setAccel( tr2i18n( "Ctrl+N" ) );
    fileOpenAction->setText( tr2i18n( "Open" ) );
    fileOpenAction->setMenuText( tr2i18n( "&Open..." ) );
    fileOpenAction->setAccel( tr2i18n( "Ctrl+O" ) );
    fileSaveAction->setText( tr2i18n( "Save" ) );
    fileSaveAction->setMenuText( tr2i18n( "&Save" ) );
    fileSaveAction->setAccel( tr2i18n( "Ctrl+S" ) );
    fileSaveAsAction->setText( tr2i18n( "Save As" ) );
    fileSaveAsAction->setMenuText( tr2i18n( "Save &As..." ) );
    fileSaveAsAction->setAccel( QString::null );
    filePrintAction->setText( tr2i18n( "Print" ) );
    filePrintAction->setMenuText( tr2i18n( "&Print..." ) );
    filePrintAction->setAccel( tr2i18n( "Ctrl+P" ) );
    fileExitAction->setText( tr2i18n( "Exit" ) );
    fileExitAction->setMenuText( tr2i18n( "E&xit" ) );
    fileExitAction->setAccel( QString::null );
    editUndoAction->setText( tr2i18n( "Undo" ) );
    editUndoAction->setMenuText( tr2i18n( "&Undo" ) );
    editUndoAction->setAccel( tr2i18n( "Ctrl+Z" ) );
    editRedoAction->setText( tr2i18n( "Redo" ) );
    editRedoAction->setMenuText( tr2i18n( "&Redo" ) );
    editRedoAction->setAccel( tr2i18n( "Ctrl+Maj+Z" ) );
    editCutAction->setText( tr2i18n( "Cut" ) );
    editCutAction->setMenuText( tr2i18n( "&Cut" ) );
    editCutAction->setAccel( tr2i18n( "Ctrl+X" ) );
    editCopyAction->setText( tr2i18n( "Copy" ) );
    editCopyAction->setMenuText( tr2i18n( "C&opy" ) );
    editCopyAction->setAccel( tr2i18n( "Ctrl+C" ) );
    editPasteAction->setText( tr2i18n( "Paste" ) );
    editPasteAction->setMenuText( tr2i18n( "&Paste" ) );
    editPasteAction->setAccel( tr2i18n( "Ctrl+V" ) );
    editFindAction->setText( tr2i18n( "Find" ) );
    editFindAction->setMenuText( tr2i18n( "&Find..." ) );
    editFindAction->setAccel( tr2i18n( "Ctrl+F" ) );
    helpContentsAction->setText( tr2i18n( "Contents" ) );
    helpContentsAction->setMenuText( tr2i18n( "&Contents..." ) );
    helpContentsAction->setAccel( QString::null );
    helpIndexAction->setText( tr2i18n( "Index" ) );
    helpIndexAction->setMenuText( tr2i18n( "&Index..." ) );
    helpIndexAction->setAccel( QString::null );
    helpAboutAction->setText( tr2i18n( "About KarthemeCreator" ) );
    helpAboutAction->setMenuText( tr2i18n( "&About KarthemeCreator" ) );
    helpAboutAction->setAccel( QString::null );
    AboutKDE->setText( tr2i18n( "About KDE" ) );
    AboutKDE->setMenuText( tr2i18n( "About &KDE" ) );
    AboutKDE->setAccel( QString::null );
    Text->setText( tr2i18n( "Text meter" ) );
    Text->setMenuText( tr2i18n( "Add Text meter" ) );
    Text->setToolTip( tr2i18n( "Add Text meter" ) );
    Image->setText( tr2i18n( "Image meter" ) );
    Image->setMenuText( tr2i18n( "Add Image meter" ) );
    Image->setToolTip( tr2i18n( "Add Image meter" ) );
    Icon->setText( tr2i18n( "Icon meter" ) );
    Icon->setMenuText( tr2i18n( "Add Icon meter" ) );
    Icon->setToolTip( tr2i18n( "Add Icon meter" ) );
    Bar->setText( tr2i18n( "Bar meter" ) );
    Bar->setMenuText( tr2i18n( "Add Bar meter" ) );
    Bar->setToolTip( tr2i18n( "Add Bar meter" ) );
    Graph->setText( tr2i18n( "Graph meter" ) );
    Graph->setMenuText( tr2i18n( "Add Graph meter" ) );
    Graph->setToolTip( tr2i18n( "Add Graph meter" ) );
    Theme->setText( tr2i18n( "Theme meter" ) );
    Theme->setMenuText( tr2i18n( "Add Theme meter" ) );
    Theme->setToolTip( tr2i18n( "Add Theme meter" ) );
    Child->setText( tr2i18n( "Child meter" ) );
    Child->setMenuText( tr2i18n( "Add Child meter" ) );
    Child->setToolTip( tr2i18n( "Add Child meter" ) );
    Font->setText( tr2i18n( "Default Font meter" ) );
    Font->setMenuText( tr2i18n( "Add Default Font meter" ) );
    Font->setToolTip( tr2i18n( "Add Default Font meter" ) );
    Click->setText( tr2i18n( "ClickArea" ) );
    Click->setMenuText( tr2i18n( "Add ClickArea" ) );
    Click->setToolTip( tr2i18n( "Add ClickArea" ) );
    Group->setText( tr2i18n( "Group" ) );
    Group->setMenuText( tr2i18n( "Add Group" ) );
    Group->setToolTip( tr2i18n( "Add Group" ) );
    Run->setText( tr2i18n( "Run theme" ) );
    Run->setToolTip( tr2i18n( "Run theme" ) );
    Stop->setText( tr2i18n( "Stop" ) );
    Stop->setToolTip( tr2i18n( "Stop" ) );
    toolBar->setLabel( tr2i18n( "Tools" ) );
    Karamba_Toolbar->setLabel( tr2i18n( "Karamba Meters" ) );
    menubar->findItem( 0 )->setText( tr2i18n( "&File" ) );
    menubar->findItem( 1 )->setText( tr2i18n( "&Edit" ) );
    menubar->findItem( 2 )->setText( tr2i18n( "Karamba" ) );
    menubar->findItem( 3 )->setText( tr2i18n( "&Help" ) );
    tabWidget->changeTab( textEdit1, tr2i18n( "Edit" ) );
    tabWidget->changeTab( preview, tr2i18n( "Preview" ) );
}

void Creator::fileNew(){
  bool Open = true;
  if( textEdit1->isModified() ){
    SaveDlg * dlg = 0;
    dlg = new SaveDlg(this, "SaveDlg", true, 0);
    dlg->setFile( currentFile );
    int result = dlg->exec();
    if(dlg->abort == false){
      if(result == QDialog::Accepted){
        fileSave();
        if( textEdit1->isModified() ){
          Open = false;
        }
      }
    }
    else{
      Open = false;
    }
  }
  if( Open ){
    NTheme* nthemedlg = 0;
    nthemedlg = new NTheme(this, "NThemeDlg", true, 0);
    if(nthemedlg->exec()){
      currentFile = tr2i18n("Untitled.theme");
      fileName = "";
      if(!nthemedlg->noHead->isChecked()){
        hasKaramba = true;
        initText();
        int X = nthemedlg->spinBox1->value();
        int Y = nthemedlg->spinBox2->value();
        int W = nthemedlg->spinBox3->value();
        int H = nthemedlg->spinBox4->value();
        int I = nthemedlg->spinBox5->value();
        int L = nthemedlg->spinBox6->value();
        QString value = "KARAMBA W=" + QString::number(W) + " H=" + QString::number(H) + " INTERVAL=" + QString::number(I);
        if(nthemedlg->checkBox7->isChecked()){
          value += " CENTER=true";
        }
        else{
          if(nthemedlg->checkBox1->isChecked()){
            value += " RIGHT=true Y=" + QString::number(Y);
          }
          else if(nthemedlg->checkBox2->isChecked()){
            value += " BOTTOM=true X=" + QString::number(X);
          }
          else if(nthemedlg->checkBox4->isChecked()){
            value += " TOPBAR=true X=" + QString::number(X);
          }
          else if(nthemedlg->checkBox5->isChecked()){
            value += " BOTTOMBAR=true X=" + QString::number(X);
          }
          else{
            value += " X=" + QString::number(X) + " Y=" + QString::number(Y);
          }
        }
        if(nthemedlg->checkBox3->isChecked()){
          value += " ONTOP=true";
        }
        if(nthemedlg->checkBox6->isChecked()){
          value += " LOCKED=true";
        }
        if(nthemedlg->Fading->isChecked()){
          value += " FADE=true";
        }
        switch( nthemedlg->hideCombo->currentItem() ){
          case 1 : value += " HIDE=LEFT";
              value += " LIMIT="+QString::number(L);
            break;
          case 2 : value += " HIDE=RIGHT";
              value += " LIMIT="+QString::number(L);
            break;
          case 3 : value += " HIDE=TOP";
              value += " LIMIT="+QString::number(L);
            break;
          case 4 : value += " HIDE=BOTTOM";
              value += " LIMIT="+QString::number(L);
            break;
        }
        value += "\n";
        addMeter(value);
      }
      else{
        hasKaramba = false;
        initText();
      }
      setModif(true);
    }
  }
}

void Creator::initText(){
    tabWidget->setEnabled( true );
    textEdit1->setPaper( QBrush( KGlobalSettings::baseColor() ) );
    textEdit1->setReadOnly( false );
    textEdit1->setText("");
    textEdit1->setEnabled( true );
    upCurPos( 0, 0 );
    Text->setEnabled(hasKaramba);
    Image->setEnabled(hasKaramba);
    Icon->setEnabled(hasKaramba);
    Bar->setEnabled(hasKaramba);
    Graph->setEnabled(hasKaramba);
    Child->setEnabled(hasKaramba);
    Font->setEnabled(hasKaramba);
    Group->setEnabled(hasKaramba);
    Click->setEnabled(hasKaramba);
    Run->setEnabled(hasKaramba);
    Theme->setEnabled(true);
    fileSaveAction->setEnabled(true);
    fileSaveAsAction->setEnabled(true);
    filePrintAction->setEnabled(true);
    editFindAction->setEnabled(true);
    editPasteAction->setEnabled(true);
    if(kp){
      if(kp->isRunning){
        kp->close();
        preview->repaint ();
        kp = 0;
      }
      //if(tabWidget->currentPageIndex() == 1){
        slotPreview(tabWidget->currentPage());
      //}
   }
}

void Creator::slotPreview( QWidget* q ){
  if(tabWidget->indexOf(q) == 1){
    if( textEdit1->isModified() ){
      QString tmpfile;
      if(fn){
        fn->unlink();
      }
      fn = new KTempFile (locateLocal("tmp", "kar"), ".theme", 0600);
      tmpfile = fn->name();
      QTextStream* ts;
      ts = fn->textStream();
      *ts << textEdit1->text();
      fn->close();
      kp = new karamba( preview, tmpfile, true );
    }
    else{
      kp = new karamba( preview, fileName, true );
    }
    kp->ChangeBgColor(paletteBackgroundColor());
    kp->slotMove(0,0);
    status->message(tr2i18n("Preview of current theme with ") + QString::number(kp->KarProcList->count()) + tr2i18n(" KarambaR child process"));
    if(kp->KarProcList->count()){
      KarPrevList = new QPtrList<karamba>();
      karamba* kc;
      int n = 0;
      for ( kc = kp->KarProcList->first(); kc; kc = kp->KarProcList->next() ){
        n++;
        kc->ChangeBgColor(paletteBackgroundColor());
        KarPrevList->append( kc );
        qDebug("Proc " + QString::number(n) + ": " +kc->themeFile);
      }
    }
    kp->show();
    if(!kp->foundKaramba){
      kp = 0;
    }
 }
  else{
    status->message(tr2i18n("Theme edition mode"));
    if(KarPrevList){
      if(KarPrevList->count()){
        karamba* kc;
        int n;
        n = 0;
        for ( kc = KarPrevList->first(); kc; kc = KarPrevList->next() ){
          n++;
          if(kc->isRunning){
            qDebug("Close child process" + QString::number(n));
            kc->close();
          }
        }
        KarPrevList = 0;
      }
    }
    if(kp){
      if(kp->isRunning){
        kp->close();
        kp = 0;
      }
    }
    preview->repaint();
  }
}

void Creator::setModif( bool m ){
    if( m ){
        setCaption( tr2i18n( "Karamba Theme Creator - " ) + currentFile + tr2i18n( " [Modified]" ) );
        textEdit1->setModified( true );
    }
    else{
        setCaption( tr2i18n( "Karamba Theme Creator - " ) + currentFile );
        textEdit1->setModified( false );
    }
    fileSaveAction->setEnabled( m );
}

void Creator::addMeter(QString v, bool cur)
{
  if(!cur){
    textEdit1->append ( v );
  }
  else{
    v += "\n";
    textEdit1->insertAt(v, line, 0);
  }
  if(kp){
    if(kp->isRunning){
      kp->close();
      preview->repaint ();
      kp = 0;
    }
    if(tabWidget->currentPageIndex() == 1){
      slotPreview(tabWidget->currentPage());
    }
  }
  setModif( true );
}

void Creator::AddIcon(){
    IconDlg* icondlg = 0;
    icondlg = new IconDlg(this, "IconDlg", true, 0);
    if(icondlg->exec()){
      int X = icondlg->X->value();
      int Y = icondlg->Y->value();
      int S = icondlg->S->value();
      int Roll = icondlg->Roll->value();
      QString value = "ICON X=" + QString::number(X) + " Y=" + QString::number(Y);
      value += " S=" + QString::number(S);
      if(Roll != 0){
        value += " ROLL=" + QString::number(Roll) + " W=" + QString::number(Roll) + " H=" + QString::number(Roll);
      }
      else{
        value += " W=" + QString::number(S) + " H=" + QString::number(S);
      }
      if(icondlg->centerCheck->isChecked()){
        value += " CENTER=true";
      }
      if(icondlg->Tooltip->text() != ""){
        value += " TOOLTIP=\"" + icondlg->Tooltip->text() + "\"";
      }
      if(icondlg->useSens->isChecked()){
        switch( icondlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + icondlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + icondlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + icondlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + icondlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + icondlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + icondlg->Program->text() + "\"";
                    value += " ENCODING=\"" + icondlg->Codec->text() + "\"";
                    if(icondlg->useSens->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + icondlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + icondlg->Program->text() + "\"";
                    value += " ENCODING=\"" + icondlg->Codec->text() + "\"";
            break;
        }
        if( ( icondlg->SensorCombo->currentItem() == 10 || icondlg->SensorCombo->currentItem() == 7 ) &&  icondlg->Line->text() != "" ){
          value += " LINE=" + icondlg->Line->text();
        }
        value += " FORMAT=\"" + icondlg->Format->text() + "\"";
      }
      else{
        value += " NAME=\"" + icondlg->icon + "\"";
      }
      addMeter(value, icondlg->insCur->isChecked());
    }
}

void Creator::AddImage(){
    ImageDlg* imagedlg = 0;
    imagedlg = new ImageDlg(this, "ImageDlg", true, 0);
    if(imagedlg->exec()){
      int X = imagedlg->X->value();
      int Y = imagedlg->Y->value();
      int W = imagedlg->W->value();
      int H = imagedlg->H->value();
      QString value = "IMAGE X=" + QString::number(X) + " Y=" + QString::number(Y);
      if(W != 0){
        value += " W=" + QString::number(W);
      }
      if(H != 0){
        value += " H=" + QString::number(H);
      }
      if(imagedlg->centerCheck->isChecked()){
        value += " CENTER=true";
      }
      if(imagedlg->Tooltip->text() != ""){
        value += " TOOLTIP=\"" + imagedlg->Tooltip->text() + "\"";
      }
      if(imagedlg->useSens->isChecked()){
        switch( imagedlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + imagedlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + imagedlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + imagedlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + imagedlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + imagedlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + imagedlg->Program->text() + "\"";
                    value += " ENCODING=\"" + imagedlg->Codec->text() + "\"";
                    if(imagedlg->useSens->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + imagedlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + imagedlg->Program->text() + "\"";
                    value += " ENCODING=\"" + imagedlg->Codec->text() + "\"";
            break;
        }
        if( ( imagedlg->SensorCombo->currentItem() == 10 || imagedlg->SensorCombo->currentItem() == 7 ) &&  imagedlg->Line->text() != "" ){
          value += " LINE=" + imagedlg->Line->text();
        }
        value += " FORMAT=\"" + imagedlg->Format->text() + "\"";
      }
      else{
        value += " PATH=\"" + imagedlg->Value->text() + "\"";
      }
      addMeter(value, imagedlg->insCur->isChecked());
    }
}

void Creator::AddClick(){
    ClickDlg* clickdlg = 0;
    clickdlg = new ClickDlg(this, "ClickDlg", true, 0);
    if(clickdlg->exec()){
      int X = clickdlg->X->value();
      int Y = clickdlg->Y->value();
      int W = clickdlg->W->value();
      int H = clickdlg->H->value();
      QString value = "CLICKAREA X=" + QString::number(X) + " Y=" + QString::number(Y);
      value += " W=" + QString::number(W) + " H=" + QString::number(H);
      if(clickdlg->Tooltip->text() != ""){
        value += " TOOLTIP=\"" + clickdlg->Tooltip->text() + "\"";
      }
      if(clickdlg->useSens->isChecked()){
        switch( clickdlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + clickdlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + clickdlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + clickdlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + clickdlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + clickdlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + clickdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + clickdlg->Codec->text() + "\"";
                    if(clickdlg->RDF->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + clickdlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + clickdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + clickdlg->Codec->text() + "\"";
            break;
        }
        if( ( clickdlg->SensorCombo->currentItem() == 10 || clickdlg->SensorCombo->currentItem() == 7 ) &&  clickdlg->Line->text() != "" ){
          value += " LINE=" + clickdlg->Line->text();
        }
        value += " FORMAT=\"" + clickdlg->Format->text() + "\"";
        value += " ONCLICK=\"" + clickdlg->OnClick->text() + "\"";
      }
      else{
        value += " ONCLICK=\"" + clickdlg->URL->text() + "\"";
      }
      addMeter(value, clickdlg->insCur->isChecked());
    }
}

void Creator::AddText(){
    TextDlg* textdlg = 0;
    textdlg = new TextDlg(this, "TextDlg", true, 0);
    if(textdlg->exec()){
      int X = textdlg->X->value();
      int Y = textdlg->Y->value();
      int W = textdlg->W->value();
      int H = textdlg->H->value();
      int fshadow = textdlg->spinBox6->value();
      QColor myColor;
      QColor myShadow;
      myColor = textdlg->textColor;
      myShadow = textdlg->shadow;
      QString value = "TEXT X=" + QString::number(X) + " Y=" + QString::number(Y);
      if(W != 0){
        value += " W=" + QString::number(W);
      }
      if(H != 0){
        value += " H=" + QString::number(H);
      }
      if(( myColor.red() + myColor.blue() + myColor.green() ) != 0){
        value += " COLOR=" + QString::number(myColor.red()) + "," + QString::number(myColor.green()) + "," + QString::number(myColor.blue());
      }
      if( fshadow != 0 ){
        value += " SHADOW=" +QString::number(fshadow);
        if(( myShadow.red() + myShadow.blue() + myShadow.green() ) != 0 ){
          value += " BGCOLOR=" + QString::number(myShadow.red()) + "," + QString::number(myShadow.green()) + "," + QString::number(myShadow.blue());
        }
      }
      if(textdlg->spinBox5->value() != 0){
        value += " FONTSIZE=" + QString::number(textdlg->spinBox5->value());
      }
      if(textdlg->FontCombo->currentText() != ""){
        value += " FONT=\"" + textdlg->FontCombo->currentText() + "\"";
      }
      switch( textdlg->comboBox2->currentItem() ){
        case 0 : value += " ALIGN=\"LEFT\"";
          break;
        case 1 : value += " ALIGN=\"CENTER\"";
          break;
        case 2 : value += " ALIGN=\"RIGHT\"";
          break;
      }
      if(textdlg->checkBox2->isChecked()){
        value += " FIXEDPITCH=true";
      }
      if(textdlg->spinBox7->value() != 0){
        value += " MOVE=" + QString::number(textdlg->spinBox7->value());
      }
      if(textdlg->spinBox8->value() != 0){
        value += " MAXMOVE=" + QString::number(textdlg->spinBox8->value());
      }
      if(textdlg->useSens->isChecked()){
        switch( textdlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + textdlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + textdlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + textdlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + textdlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + textdlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + textdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + textdlg->Codec->text() + "\"";
                    if(textdlg->RDF->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + textdlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + textdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + textdlg->Codec->text() + "\"";
            break;
        }
        if( ( textdlg->SensorCombo->currentItem() == 10 || textdlg->SensorCombo->currentItem() == 7 ) &&  textdlg->Line->text() != "" ){
          value += " LINE=" + textdlg->Line->text();
        }
        value += " FORMAT=\"" + textdlg->Format->text() + "\"";
      }
      else{
        value += " VALUE=\"" + textdlg->Value->text() + "\"";
      }
      addMeter(value, textdlg->insCur->isChecked());
    }
}

void Creator::AddBar(){
    BarDlg* bardlg = 0;
    bardlg = new BarDlg(this, "BarDlg", true, 0);
    if(bardlg->exec()){
      int X =bardlg->X->value();
      int Y = bardlg->Y->value();
      int W = bardlg->W->value();
      int H = bardlg->H->value();
      int min = bardlg->Min->value();
      int max = bardlg->Max->value();
      QString value = "BAR X=" + QString::number(X) + " Y=" + QString::number(Y);
      value += " PATH=\"" + bardlg->Pix->text() + "\"";
      if(W != 0){
        value += " W=" + QString::number(W);
      }
      if(H != 0){
        value += " H=" + QString::number(H);
      }
      if(min != 0){
        value += " MIN=" + QString::number(min);
      }
      if(max != 100){
        value += " MAX=" + QString::number(max);
      }
      if(bardlg->Vertical->isChecked()){
        value += " VERTICAL=true";
      }
      if(bardlg->useSens->isChecked()){
        switch( bardlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + bardlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + bardlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + bardlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + bardlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + bardlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + bardlg->Program->text() + "\"";
                    value += " ENCODING=\"" + bardlg->Codec->text() + "\"";
                    if(bardlg->RDF->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + bardlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + bardlg->Program->text() + "\"";
                    value += " ENCODING=\"" + bardlg->Codec->text() + "\"";
            break;
        }
        if( ( bardlg->SensorCombo->currentItem() == 10 || bardlg->SensorCombo->currentItem() == 7 ) &&  bardlg->Line->text() != "" ){
          value += " LINE=" + bardlg->Line->text();
        }
        value += " FORMAT=\"" + bardlg->Format->text() + "\"";
      }
      else if(bardlg->Value->value() != 0){
        value += " VALUE=" + QString::number(bardlg->Value->value());
      }
      addMeter(value, bardlg->insCur->isChecked());
    }
}

void Creator::AddGraph(){
    GraphDlg* graphdlg = 0;
    graphdlg = new GraphDlg(this, "GraphDlg", true, 0);
    if(graphdlg->exec()){
      int X =graphdlg->spinBox1->value();
      int Y = graphdlg->spinBox2->value();
      int W = graphdlg->spinBox1_2->value();
      int H = graphdlg->spinBox1_3->value();
      int min = graphdlg->spinBox1_4->value();
      int max = graphdlg->spinBox1_5->value();
      int pts = graphdlg->spinBox1_6->value();
      QColor myColor;
      myColor = graphdlg->graphColor;
      QString value = "GRAPH X=" + QString::number(X) + " Y=" + QString::number(Y);
      if(pts != 50){
        value += " POINTS=" + QString::number(pts);
      }
      if(W != 0){
        value += " W=" + QString::number(W);
      }
      if(H != 0){
        value += " H=" + QString::number(H);
      }
      if(min != 0){
        value += " MIN=" + QString::number(min);
      }
      if(max != 100){
        value += " MAX=" + QString::number(max);
      }
      if(graphdlg->useSens->isChecked()){
        switch( graphdlg->SensorCombo->currentItem() ){
            case 0 : value += " SENSOR=CPU";
                      value += " CPU=\"" + graphdlg->Line->text() + "\"";
              break;
            case 1 : value += " SENSOR=Memory";
              break;
            case 2 : value += " SENSOR=Disk";
                      value += " MOUNTPOINT=\"" + graphdlg->Program->text() + "\"";
              break;
            case 3 : value += " SENSOR=Network";
                      value += " DEVICE=\"" + graphdlg->Program->text() + "\"";
                      value += " DECIMALS=\"" + graphdlg->Line->text() + "\"";
              break;
            case 4 : value += " SENSOR=Time";
              break;
            case 5 : value += " SENSOR=Uptime";
              break;
            case 6 : value += " SENSOR=Sensor";
                      value += " TYPE=\"" + graphdlg->Program->text() + "\"";
              break;
            case 7 : value += " SENSOR=Textfile";
                      value += " PATH=\"" + graphdlg->Program->text() + "\"";
                      value += " ENCODING=\"" + graphdlg->Codec->text() + "\"";
                      if(graphdlg->RDF->isChecked()){
                        value += " RDF=\"true\"";
                      }
              break;
            case 8 : value += " SENSOR=Noatun";
              break;
            case 9 : value += " SENSOR=XMMS";
                      value += " ENCODING=\"" + graphdlg->Codec->text() + "\"";
              break;
            case 10 : value += " SENSOR=Program";
                      value += " PROGRAM=\"" + graphdlg->Program->text() + "\"";
                      value += " ENCODING=\"" + graphdlg->Codec->text() + "\"";
              break;
        }
        if( ( graphdlg->SensorCombo->currentItem() == 10 || graphdlg->SensorCombo->currentItem() == 7 ) &&  graphdlg->Line->text() != "" ){
          value += " LINE=" + graphdlg->Line->text();
        }
        value += " FORMAT=\"" + graphdlg->Format->text() + "\"";
      }
      value += " COLOR=" + QString::number(myColor.red()) + "," + QString::number(myColor.green()) + "," + QString::number(myColor.blue());
      addMeter(value, graphdlg->insCur->isChecked());
    }
}

void Creator::AddTheme(){
    KSimpleConfig conf("karambarc",true);
    conf.setGroup("karamba");
    QString Dir = conf.readEntry("ThemeDir", locateLocal("data", "karamba/themes/"));
    QString fileName;
    fileName = KFileDialog::getOpenFileName(Dir, "*.theme", 0, "Add Theme");
    if( fileName != ""){
      QString value = "THEME PATH=\"" + fileName + "\"";
      addMeter(value);
    }
}

void Creator::AddChild(){
    ChildDlg* childdlg = 0;
    childdlg = new ChildDlg(this, "ChildDlg", true, 0);
    if(childdlg->exec()){
      int X =childdlg->X->value();
      int Y = childdlg->Y->value();
      int W = childdlg->W->value();
      int H = childdlg->H->value();
      QString value = "CHILD X=" + QString::number(X) + " Y=" + QString::number(Y);
      if(W != 0){
        value += " W=" + QString::number(W);
      }
      if(H != 0){
        value += " H=" + QString::number(H);
      }
      switch( childdlg->shEvents->currentItem() ){
        case 0 : value += " MOUSE=\"over\"";
          break;
        case 1 : value += " MOUSE=\"click\"";
          break;
        case 2 : value += " MOUSE=\"ofocus\"";
          break;
        case 3 : value += " MOUSE=\"cfocus\"";
          break;
      }
      if(childdlg->embed->isChecked()){
        value += " EMBED=\"true\"";
      }
      if(childdlg->Tooltip->text() != ""){
        value += " TOOLTIP=\"" + childdlg->Tooltip->text() + "\"";
      }
      if(childdlg->useSens->isChecked()){
        switch( childdlg->SensorCombo->currentItem() ){
          case 0 : value += " SENSOR=CPU";
                    value += " CPU=\"" + childdlg->Line->text() + "\"";
            break;
          case 1 : value += " SENSOR=Memory";
            break;
          case 2 : value += " SENSOR=Disk";
                    value += " MOUNTPOINT=\"" + childdlg->Program->text() + "\"";
            break;
          case 3 : value += " SENSOR=Network";
                    value += " DEVICE=\"" + childdlg->Program->text() + "\"";
                    value += " DECIMALS=\"" + childdlg->Line->text() + "\"";
            break;
          case 4 : value += " SENSOR=Time";
            break;
          case 5 : value += " SENSOR=Uptime";
            break;
          case 6 : value += " SENSOR=Sensor";
                    value += " TYPE=\"" + childdlg->Program->text() + "\"";
            break;
          case 7 : value += " SENSOR=Textfile";
                    value += " PATH=\"" + childdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + childdlg->Codec->text() + "\"";
                    if(childdlg->RDF->isChecked()){
                      value += " RDF=\"true\"";
                    }
            break;
          case 8 : value += " SENSOR=Noatun";
            break;
          case 9 : value += " SENSOR=XMMS";
                    value += " ENCODING=\"" + childdlg->Codec->text() + "\"";
            break;
          case 10 : value += " SENSOR=Program";
                    value += " PROGRAM=\"" + childdlg->Program->text() + "\"";
                    value += " ENCODING=\"" + childdlg->Codec->text() + "\"";
            break;
        }
        if( ( childdlg->SensorCombo->currentItem() == 10 || childdlg->SensorCombo->currentItem() == 7 ) &&  childdlg->Line->text() != "" ){
          value += " LINE=" + childdlg->Line->text();
        }
        value += " FORMAT=\"" + childdlg->Format->text() + "\"";
      }
      else{
       value += " SRC=\"" + childdlg->Theme->text() + "\"";
      }
      addMeter(value, childdlg->insCur->isChecked());
    }
}

void Creator::AddGroup(){
  bool txtsel = textEdit1->hasSelectedText();
  QString sel = textEdit1->selectedText();
  GroupDlg* groupdlg = 0;
  groupdlg = new GroupDlg(this, "GroupDlg", true, 0);
  if(groupdlg->exec()){
    if(txtsel == true){
      QString tmpstr;
      QString newsel;
      tmpstr = textEdit1->text();
      if(sel.right(1) != "\n"){
        newsel = "<GROUP> X=" + QString::number(groupdlg->spinBox1->value()) + " Y=" + QString::number(groupdlg->spinBox2->value()) + "\n" + sel + "\n</GROUP>" ;
      }
      else{
        newsel = "<GROUP> X=" + QString::number(groupdlg->spinBox1->value()) + " Y=" + QString::number(groupdlg->spinBox2->value()) + "\n" + sel + "</GROUP>\n" ;
      }
      tmpstr.replace( QRegExp(sel, false), newsel);
      textEdit1->setText(tmpstr);
      setModif( true );
    }
    else{
      textEdit1->append( "<GROUP> X=" + QString::number(groupdlg->spinBox1->value()) + " Y=" + QString::number(groupdlg->spinBox2->value()) + "\n</GROUP>" );
      setModif( true );
    }
  }
}

void Creator::AddFont(){
    FontDlg* fontdlg = 0;
    fontdlg = new FontDlg(this, "FontDlg", true, 0);
    if(fontdlg->exec()){
      int fsize = fontdlg->spinBox5->value();
      int fshadow = fontdlg->spinBox6->value();
      QColor myColor;
      QColor myShadow;
      myColor = fontdlg->textColor;
      myShadow = fontdlg->shadow;
      QString value = "DEFAULTFONT";
      if(( myColor.red() + myColor.blue() + myColor.green() ) != 0){
        value += " COLOR=" + QString::number(myColor.red()) + "," + QString::number(myColor.green()) + "," + QString::number(myColor.blue());
      }
      if( fshadow != 0 ){
        value += " SHADOW=" +QString::number(fshadow);
        if(( myShadow.red() + myShadow.blue() + myShadow.green() ) != 0 ){
          value += " BGCOLOR=" + QString::number(myShadow.red()) + "," + QString::number(myShadow.green()) + "," + QString::number(myShadow.blue());
        }
      }
      if(fontdlg->comboBox3->currentText() != ""){
        value += " FONT=\"" + fontdlg->comboBox3->currentText() + "\"";
      }
      switch( fontdlg->comboBox2->currentItem() ){
        case 0 : value += " ALIGN=\"LEFT\"";
          break;
        case 1 : value += " ALIGN=\"CENTER\"";
          break;
        case 2 : value += " ALIGN=\"RIGHT\"";
          break;
      }
      if(fontdlg->checkBox2->isChecked() == true){
        value += " FIXEDPITCH=true";
      }
      if(fsize != 0){
        value += " FONTSIZE=" +QString::number(fsize);
      }
      addMeter(value, fontdlg->insCur->isChecked());
    }
}

void Creator::fileOpen(QString fn){
  bool Open = true;
  if( textEdit1->isModified() ){
    SaveDlg * dlg = 0;
    dlg = new SaveDlg(this, "SaveDlg", true, 0);
    dlg->setFile( currentFile );
    int result = dlg->exec();
    if(dlg->abort == false){
      if(result == QDialog::Accepted){
        fileSave();
        if( textEdit1->isModified() ){
          Open = false;
        }
      }
    }
    else{
      Open = false;
    }
  }
  if( Open ){
    if( k ){
      if(k->isRunning){
        stopKaramba();
      }
    }
    if( fn == 0 ){
      KSimpleConfig conf("karambarc",true);
      conf.setGroup("karamba");
      QString Dir = conf.readEntry("ThemeDir", locateLocal("data", "karamba/themes/"));
      fileName = KFileDialog::getOpenFileName(Dir, "*.theme", 0, "Open configurations");
    }
    else{
      fileName = fn;
    }
    QFile file( fileName ); // Read the text from a file
    if ( file.open( IO_ReadOnly ) ) {
        QTextStream stream( &file );
        QString firstLine = stream.readLine();
        if(firstLine.find("karamba", 0, false) != -1){
          hasKaramba = true;
        }
        else{
          hasKaramba = false;
        }
        initText();
        textEdit1->setText( firstLine + "\n" + stream.read() );
        QFileInfo fi(fileName);
        currentFile = fi.fileName();
        setModif( false );
        parse();
    }
  }
}

void Creator::fileSave()
{
    if(currentFile == "Untitled.theme"){
      KSimpleConfig conf("karambarc",true);
      conf.setGroup("karamba");
      QString Dir = conf.readEntry("ThemeDir", locateLocal("data", "karamba/themes/"));
      fileName = KFileDialog::getSaveFileName(Dir, "*.theme", 0, "Save configurations");
    }
    if(fileName != ""){
      QFile file( fileName ); // Write the text to a file
      if ( file.open( IO_WriteOnly ) ) {
          QTextStream stream( &file );
          stream << textEdit1->text();
          QFileInfo fi(fileName);
          currentFile = fi.fileName();
          setModif( false );
      }
    }
}

void Creator::fileSaveAs(){
    QString nfileName;
    KSimpleConfig conf("karambarc",true);
    conf.setGroup("karamba");
    QString Dir = conf.readEntry("ThemeDir", locateLocal("data", "karamba/themes/"));
    nfileName = KFileDialog::getSaveFileName(Dir, "*.theme", 0, "Save configurations");
    if(nfileName != ""){
      QFile file( nfileName ); // Write the text to a file
      if ( file.open( IO_WriteOnly ) ) {
          QTextStream stream( &file );
          stream << textEdit1->text();
          QFileInfo fi(nfileName);
          currentFile = fi.fileName();
          fileName = nfileName;
          setModif( false );
      }
    }
}

void Creator::filePrint(){
    qWarning( "Creator::filePrint(): Not implemented yet" );
}

void Creator::fileExit(){
  close();
}

void Creator::closeEvent( QCloseEvent* ce ){
  if( textEdit1->isModified() == true ){
    SaveDlg * dlg = 0;
    dlg = new SaveDlg(this, "SaveDlg", true, 0);
    dlg->setFile( currentFile );
    int result = dlg->exec();
    if(dlg->abort == false){
      if(result == QDialog::Accepted){
        fileSave();
      }
      ce->accept( );
      qApp->quit();
    }
    else{
      ce->ignore();
    }
  }
  else{
    ce->accept( );
    qApp->quit();
  }
}

void Creator::editUndo(){
    textEdit1->undo();
}

void Creator::editRedo(){
    textEdit1->redo();
}

void Creator::editCut(){
    textEdit1->cut();
}

void Creator::editCopy(){
    textEdit1->copy();
}

void Creator::editPaste(){
    textEdit1->paste();
}

void Creator::editFind(){
}

void Creator::slotFindNext(){
}

void Creator::helpIndex(){
  KRun::runCommand( "khelpcenter help:/" );
}

void Creator::helpContents(){
  KRun::runCommand( "khelpcenter help:/karamba" );
}

void Creator::helpAbout(){
    KAboutApplication* aboutdlg;
    aboutdlg = new KAboutApplication();
    aboutdlg->show();
}

void Creator::aboutKDE(){
    KAboutKDE* aboutkdedlg;
    aboutkdedlg = new KAboutKDE();
    aboutkdedlg->show();
}

void Creator::RunTheme(){
  if( textEdit1->isModified() ){
    if( k ){
      qDebug("hahahahah --- K not Null");
      if(k->isRunning == true){
        qDebug("hahahahah --- Stop K");
        stopKaramba();
      }
    }
    QString tmpfile;
    if(fn){
      fn->unlink();
    }
    fn = new KTempFile (locateLocal("tmp", "kar"), ".theme", 0600);
    tmpfile = fn->name();
    QTextStream* ts;
    ts = fn->textStream();
    *ts << textEdit1->text();
    qDebug("--- ");
    fn->close();
    qDebug("hahahahah --- Knew");
    k = new karamba( 0, tmpfile, true );
    status->message(tr2i18n("Running of current theme with ") + QString::number(k->KarProcList->count()) + tr2i18n(" KarambaR child process"));
    if(k->KarProcList->count()){
      KarRunList = new QPtrList<karamba>();
      karamba* kc;
      int n = 0;
      for ( kc = k->KarProcList->first(); kc; kc = k->KarProcList->next() ){
        n++;
        KarRunList->append( kc );
        qDebug("Proc " + QString::number(n) + ": " +kc->themeFile);
      }
    }
    connect( k, SIGNAL( karambaRun( bool ) ), this, SLOT( slotRunTest( bool ) ) );
    slotRunTest(true);
    k->show();
  }
  else{
    if( k ){
      if(k->isRunning){
        k->reloadConfig();
        qDebug("hahahahah --- Refresh K");
      }
    }
    else{
      k = new karamba( 0, fileName, true );
      status->message(tr2i18n("Running of current theme with ") + QString::number(k->KarProcList->count()) + tr2i18n(" KarambaR child process"));
      if(k->KarProcList->count()){
        KarRunList = new QPtrList<karamba>();
        karamba* kc;
        int n = 0;
        for ( kc = k->KarProcList->first(); kc; kc = k->KarProcList->next() ){
          n++;
          KarRunList->append( kc );
          qDebug("Proc " + QString::number(n) + ": " +kc->themeFile);
        }
      }
      connect( k, SIGNAL( karambaRun( bool ) ), this, SLOT( slotRunTest( bool ) ) );
      slotRunTest(true);
      k->show();
      qDebug("hahahahah --- run K");
    }
  }
}

void Creator::stopKaramba(){
  k->close();
  disconnect(k);
  k = 0;
  if(fn){
    fn->unlink();
    fn = 0;
  }
}

void Creator::slotRunTest( bool b ){
  if(b){
    Run->setText( tr2i18n( "Refresh theme" ) );
    Run->setToolTip( tr2i18n( "Refresh theme" ) );
    Run->setIconSet( SmallIconSet( "reload" ));
    Stop->setEnabled( true );
  }
  else{
    Run->setText( tr2i18n( "Run theme" ) );
    Run->setToolTip( tr2i18n( "Run theme" ) );
    Run->setIconSet( SmallIconSet( "run" ));
    Stop->setEnabled( false );
  }
}

void Creator::upCurPos( int para, int pos ){
  line = para;
  row = pos;
  status->changeItem (tr2i18n("Line: ")+QString::number(para + 1), 1);
  status->changeItem (tr2i18n("Col: ")+QString::number(pos), 2);
  textEdit1->clearParagraphBackground(prevpara);
  textEdit1->setParagraphBackgroundColor(para, KGlobalSettings::alternateBackgroundColor());;
  prevpara = para;
  parse();
}

void Creator::parse(){
  int p;
  QString l;
  for(p = 0; p <= textEdit1->paragraphs(); p++){
    l = textEdit1->text(p);
    if(l.left(1) == "#"){
      textEdit1->setParagraphBackgroundColor(p, QColor(230, 230, 230));;
    }
  }
}

#include "theme_creator.moc"
