#ifndef KARAMBA_CONF_H
#define KARAMBA_CONF_H

#include <qvariant.h>
#include <qpixmap.h>
#include <qdialog.h>
#include <qvbox.h>
#include <qurl.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qcheckbox.h>
#include <qspinbox.h>

#include <ktextedit.h>
#include <kiconloader.h>
#include <ksimpleconfig.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <krun.h>
#include <kdirselectdialog.h>
#include <klocale.h>
#include <kstringhandler.h>
#include <kstandarddirs.h>
#include <kcolordialog.h>
#include <kselect.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QGroupBox;
class QIconView;
class QIconViewItem;
class QLabel;
class QLineEdit;
class QPushButton;
class QTabWidget;
class QWidget;
class QCheckBox;
class QSpinBox;
class QGridLayout;

/**
 * @short Application Configuration Window
 * @author Bosselut Cyril <bosselut@b1project.com>
 * @version 0.23
 */
class KarambaConf : public QDialog
{
    Q_OBJECT

public:
    KarambaConf( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~KarambaConf();

    QPushButton* buttonHelp;
    QPushButton* buttonOk;
    QPushButton* buttonCancel;
    QTabWidget* tabWidget;
    QWidget* Widget2;
    QGroupBox* dFolderGroup;
    QLineEdit* dFolder;
    QPushButton* browse_1;
    QGroupBox* dThemeGroup;
    QLineEdit* dTheme;
    QLabel* themeLabel;
    QPushButton* browse_2;
    QGroupBox* dAppsGroup;
    QLineEdit* dViewer;
    QPushButton* browse_3;
    QLabel* viewerLabel;
    QWidget* Widget3;
    QGroupBox* CoversGroup;
    QGroupBox* TagGroup;
    QGroupBox* XmmsGroup;
    QLabel* textLabel1;
    QLineEdit* cWidth;
    QLineEdit* cHeight;
    QLabel* textLabel2;
    QCheckBox* id3Check;
    QCheckBox* SaveId3Check;
    QCheckBox* CoverCheck;
    QCheckBox* id3CoverCheck;
    QCheckBox* NetCoverCheck;
    QCheckBox* mainWinCheck;
    QWidget* Widget4;
    QLabel* textLabel3;
    QPushButton* colorButton;
    QGroupBox* EmbedGroup;
    QColor bgColor;
    QGroupBox* FadeGroup;
    QLabel* textLabel4;
    QSpinBox* fadeSpin;
    double fadeLevel;
    QLabel* textLabel5;
    QLabel* textLabel6;
    QLabel* textLabel7;
    QPushButton* fcolorButton;
    QColor fadeColor;

protected:
    QHBoxLayout* Layout1;
    QHBoxLayout* layout5;
    QVBoxLayout* layout3;
    QGridLayout* Layout2;
    QGridLayout* Layout4;
    QGridLayout* Layout6;

protected slots:
    virtual void languageChange();
    virtual void selColor();
    virtual void selFadeColor();

private:
  QString Dir;
  QString Theme;
  QString Viewer;
  QString bgc;
  QString fadec;
  int CoverWidth;
  int CoverHeight;
  bool EnableID3;
  bool SaveID3;
  bool EnableCover;
  bool EnableId3Cover;
  bool NetCover;
  bool HideMainWin;

private slots:
    void loadConfig();
    void saveConfig();
    void selThemeDir();
    void selDefaultTheme();
    void selDefaultViewer();
    void toggleCover();
    void help();
    void slotTestFade( int f );

};

#endif // KARAMBA_CONF_H
