//
//
// C++ Implementation: $MODULE$
//
// Description: 
//
//
// Author: Bosselut Cyril <bosselut@b1project.com>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tags.h"

Tags::Tags( QString fn ){
  file = fn;
}


Tags::~Tags(){
}

void Tags::setInfos(QString tr, QString ar, QString ti, QString al){
  track = tr;
  artist = ar;
  stitle = ti;
  album = al;
}

bool Tags::isMP3(){
  KMimeMagicResult *result = KMimeMagic::self()->findFileType(file);
  if(result->mimeType() == "audio/x-mp3"){
    return true;
  }
  else{
    return false;
  }
}

void Tags::ReadId3(){
  myTag = new ID3_Tag();
  myTag->Link(file, ID3TT_ALL);
  const Mp3_Headerinfo * mp3header = myTag->GetMp3HeaderInfo();
  rate = mp3header->bitrate;
  int chmode = mp3header->channelmode;
  if(chmode == 0 || chmode == 1 || chmode == 2){
    channel = 2;
  }
  else if(chmode == 3 || chmode == 4){
    channel = 1;
  }
  else{
    channel = 0;
  }
  int tmpfreq = mp3header->frequency;
  freq = ((QString::number(tmpfreq).toDouble())/1000);
  kdDebug("Tags") << "Getting tags from MP3 file" << endl;
  stitle = upFirst(ID3_GetTitle(myTag));
  artist = upFirst(ID3_GetArtist(myTag));
  album = upFirst(ID3_GetAlbum(myTag));
  track = ID3_GetTrack(myTag);
  if(stitle != ""){
    title = artist + " - " + stitle + " - " + album;
  }
  myTag = 0;
}

bool Tags::id3cover(int w, int h){
  myTag = new ID3_Tag();
  myTag->Link(file, ID3TT_ALL);
  ID3_Frame* myFrame = myTag->Find(ID3FID_PICTURE);// Check if id3 have PICTURE field
  if (NULL != myFrame){
    kdDebug("Tags") << "id3 have picture " << endl;
    fp = locateLocal("tmp", "cover.png");
    ID3_Field* myField = myFrame->GetField(ID3FN_DATA);
    const uchar* data = myField->GetRawBinary();
    size_t size = myField->Size();
    QImage img;
    QImage tmpimg;
    img.loadFromData (data, size);
    if(img.width() >= w || img.height() >= h){ //scale image if necessary
      tmpimg = img.scale(h, w, QImage::ScaleMin);
    }
    tmpimg.save(fp, "PNG", 0);
    img.save( locateLocal("tmp", "rcover.png"), "PNG", 0);
    img.reset();
    tmpimg.reset();
    return true;
  }
  else{
    kdDebug("Tags") << "id3 don't have picture " << endl;
    return false;
  }
  myTag = 0;
}

void Tags::WriteId3(){
  kdDebug("Tags") << "Adding id3 tags to file " << file << endl;
  myTag = new ID3_Tag();
  myTag->Link(file, ID3TT_ALL);
  ID3_AddTitle(myTag, stitle, true);
  ID3_AddAlbum(myTag, album, true);
  ID3_AddArtist(myTag, artist, true);
  ID3_AddTrack(myTag, track.latin1() , '0', true);
  myTag->Update();
  myTag = 0;
}

bool Tags::isOGG(){
  KMimeMagicResult *result = KMimeMagic::self()->findFileType(file);
  if(result->mimeType() == "application/x-ogg"){
    return true;
  }
  else{
    return false;
  }
}

void Tags::ReadVorbis(){
  OggVorbis_File  o;
  FILE *f = fopen( QFile::encodeName(file), "rb");
  int t = ov_open(f, &o, NULL, 0);
  if(t == 0){
    // this part comes from KTagit http://ktagit.sourceforge.net/
    char **ptr = ov_comment(&o, -1)->user_comments;
    kdDebug("Tags") << "Getting tags from OGG file" << endl;
    for(; *ptr; ptr++){
        QString tmp(QString::fromUtf8(*ptr));
        if(tmp.find('=')){
          QString tag (tmp.section('=', 0, 0).lower());
          QString value (tmp.section('=', 1, 1));
          if(!tag.compare("title")){
            stitle = upFirst(value);
          }
          if(!tag.compare("artist")){
            artist = upFirst(value);
          }
          if(!tag.compare("album")){
            album = upFirst(value);
          }
          if(tag.contains("track")){
            track = value;
          }
        }
    }
    //---//
    if(stitle != ""){
      title = artist + " - " + stitle + " - " + album;
      rate = ov_bitrate(&o, -1);
    }
    ov_clear(&o);
  }
  else{
    kdError("Tags") << "ReadVorbis() " << t << endl;
    fclose(f);
  }
}

void Tags::WriteVorbis(){
  kdDebug("Tags") << "Adding OGG tags to file " << file << endl;
  KFileMetaInfo* info = new KFileMetaInfo ( file );
  if(!info->containsGroup("Comment")){
    info->addGroup("Comment");
    kdDebug("Tags") << "Adding Comment group" << endl;
  }
  KFileMetaInfoGroup group = info->group("Comment");
  if(group.isEmpty()){
    group.addItem("Title");
    group.addItem("Artist");
    group.addItem("Album");
    group.addItem("Track");
    group.addItem("Tracknumber");
    kdDebug("Tags") << "Adding items to group Comment" << endl;
  }
  if((info->item("Title").value()).isNull()){
    info->item("Title").setValue(stitle);
  }
  if((info->item("Artist").value()).isNull()){
    info->item("Artist").setValue(artist);
  }
  if((info->item("Album").value()).isNull()){
    info->item("Album").setValue(album);
  }
  if((info->item("Track").value()).isNull()){
    info->item("Track").setValue(track);
  }
  if((info->item("Tracknumber").value()).isNull()){
    info->item("Tracknumber").setValue(track);
  }
  info->applyChanges();
}

bool Tags::darkcover(QString dir){
  kdDebug("Tags") << "Searching cover on darktown.com" << endl;
  QString ar = (artist.lower()).replace(" ", "_");
  QString al = (album.lower()).replace(" ", "_");
  QString l = ar.left(1);
  QString out = dir+"/cover.jpg";
  QString mime = "";
  KURL u1("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+"-_"+al+"a.jpg");
  mime = KIO::NetAccess::mimetype( u1 );
  if( mime == "image/jpeg" ){
    kdDebug("Tags") << "Cover found, downloading" << endl;
    return KIO::NetAccess::download( u1, out );
  }
  else{
    KURL u2("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+al+"a.jpg");
    mime = KIO::NetAccess::mimetype( u2 );
    if( mime == "image/jpeg" ){
      kdDebug("Tags") << "Cover found, downloading" << endl;
      return KIO::NetAccess::download( u2, out );
    }
    else{
      QString ar = (upFirst(artist.lower())).replace(" ", "_");
      QString al = (upFirst(album.lower())).replace(" ", "_");
      KURL u3("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+al+"a.jpg");
      mime = KIO::NetAccess::mimetype( u3 );
      if( mime == "image/jpeg" ){
        kdDebug("Tags") << "Cover found, downloading" << endl;
        return KIO::NetAccess::download( u3, out );
      }
      else{
        kdDebug("Tags") << "Cover not found" << endl;
        return false;
      }
    }
  }
}

QString Tags::setCover(int w, int h, QString cover){
  fp = locateLocal("tmp", "cover.png");
  QImage img;
  QImage tmpimg;
  img.load(cover);
  if(img.width() >= w || img.height() >= h){ //scale image if necessary
    tmpimg = img.smoothScale(h, w, QImage::ScaleMin);
    tmpimg.save(fp, "PNG", 0);
    img.reset();
    tmpimg.reset();
  }
  else{
    fp = cover;
  }
  return fp;
}

QString Tags::upFirst( QString str){
  QStringList ls = 0;
  QString upstr = 0;
  ls = QStringList::split( ' ', str);
  QStringList::Iterator it = ls.begin();
  while( it!= ls.end() ){
    QString word = *it;
    QChar i = word.at(0);
    word.replace ( 0, 1, i.upper() );
    upstr += word + " " ;
    ++it;
  }
  if(upstr.right(3) == " - "){
    upstr = upstr.remove(upstr.length() - 3, 3);
  }
  return upstr;
}

