/***************************************************************************
*   Copyright (C) 2003 by Hans Karlsson                                   *
*   karlsson.h@home.se                                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#include "xmmssensor.h"

XMMSSensor::XMMSSensor( int interval, QString encoding, int nw, int nh, karamba* k ) : Sensor( interval )
{
     if( encoding != "" )
    {
        codec = QTextCodec::codecForName( encoding );
        if ( codec == 0)
            codec = QTextCodec::codecForLocale();
    }
    else{
        codec = QTextCodec::codecForLocale();
    }
    slotLoadConfig();
    w = getW( nw );
    h = getH( nh );
    current = 0;
    pos = 0;
    title = "";
    stitle = "";
    artist = "";
    album = "";
    track = "";
    vol = 0;
    currentTime = 0;
    setKaramba(k);
    connect( this,SIGNAL(songChange()), this,SLOT(slotUpValues()) );
    connect( myKaramba,SIGNAL(configChanged()), this,SLOT(slotLoadConfig()) );
#ifdef HAVE_XMMS
    if(xmms_remote_is_running(0)){
      slotGetPos();
    }
#endif // HAVE_XMMS
}

XMMSSensor::~XMMSSensor()
{}

int XMMSSensor::getW( int nw ){
  if(nw == 0){
    nw = defW;
  }
   return nw;
}

int XMMSSensor::getH( int nh ){
  if(nh == 0){
    nh = defH;
  }
  return nh;
}

void XMMSSensor::slotUpValues(){
#ifdef HAVE_XMMS
    freq = 0;
    channel = 0;
    rate = 0;
    kdDebug("XMMS sensor") << "Song change, updating values" << endl;
    title = "";
    stitle = "";
    artist = "";
    album = "";
    track = "";
    cover = "";
    rcover = "";
    file = codec->toUnicode( QCString( xmms_remote_get_playlist_file( 0, pos ) ) ); //get the path and filename
    Tags* tags = new Tags( file );
    if(tags->isMP3() && EnableID3){ // Check if file is an mp3
      tags->ReadId3();
      stitle = tags->stitle;
      artist = tags->artist;
      album = tags->album;
      track = tags->track;
      title = tags->title;
      rate = tags->rate;
      channel = tags->channel;
      freq = tags->freq;
    }
    else if(tags->isOGG() && EnableID3){ // Check if file is an ogg
      tags->ReadVorbis();
      stitle = tags->stitle;
      artist = tags->artist;
      album = tags->album;
      track = tags->track;
      title = tags->title;
      rate = tags->rate;
      int tmpfreq;
      xmms_remote_get_info(0, &rate, &tmpfreq, &channel);
      freq = ((QString::number(tmpfreq).toDouble())/1000);
    }
    if( title == "" ){
      kdDebug("XMMS sensor") << "Getting tags from XMMS" << endl;
      title = codec->toUnicode( QCString( xmms_remote_get_playlist_title( 0, pos ) ) );
      title = tags->upFirst(title);
      int c = title.contains(" - ");
      if(c == 3){
      /**
      * this works only if you set
      * XMMS to display song informations like this:
      * %n - %p - %t - %a
      * this is an automatic detection, so the values may be incorrect.
      */
        track = (title.section("- ", 0, 0)).stripWhiteSpace();
        artist = (title.section("- ", 1, 1)).stripWhiteSpace();
        stitle = (title.section("- ", 2, 2)).stripWhiteSpace();
        album = ((title.section("- ", 3, 3)).stripWhiteSpace()).replace(".ogg","");
      }
      else if(c == 1){
      /**
      * this case file name is "track - stitle.{ogg,mp3}"
      */
        track = (title.section("- ", 0, 0)).stripWhiteSpace();
        stitle = ((title.section("- ", 1, 1)).stripWhiteSpace()).replace(".ogg","");
      }
      else{
      /**
      * this case file name is "stitle.{ogg,mp3}"
      */
        stitle = title.replace(".ogg","");
      }
      tags->setInfos(track, artist, stitle, album);
      if(SaveID3 && QFileInfo(file).isWritable()){ // Check if Karamba can write tags
        if(tags->isMP3()){ // Check if file is an mp3
          tags->WriteId3();
        }
        if(tags->isOGG()){ // Check if file is an ogg
          tags->WriteVorbis();
        }
      }
      int tmpfreq;
      xmms_remote_get_info(0, &rate, &tmpfreq, &channel);
      freq = ((QString::number(tmpfreq).toDouble())/1000);
    }
    kdDebug("XMMS sensor") << title << endl;
    if(channel == 1){
      monoster = "mono";
    }
    else if(channel == 2){
      monoster = "stereo";
    }
    else{
      monoster = "nosound";
    }
    songLength = xmms_remote_get_playlist_time( 0, pos );
    if(EnableCover){
      slotGetCover( tags );
    }
    tags = 0;
#endif // HAVE_XMMS
}

void XMMSSensor::slotGetCover( Tags* tags ){
  bool id3c = false;
  if(tags->isMP3() && EnableID3C){ // Check if file is an mp3
    id3c = tags->id3cover(w, h);
    if(id3c){
      cover = locateLocal("tmp", "cover.png");
      rcover = locateLocal("tmp", "rcover.png");
    }
  }
  if(!id3c){ // File is not an mp3 or don't have id3 PICTURE field
    QUrl url(file); //make QUrl from QString
    dir = url.dirPath (); //resolve the absolute path
    if(QFile::exists(dir + "/cover.png")){
      rcover = dir + "/cover.png";
      cover = tags->setCover(w, h, rcover);
    }
    else if(QFile::exists(dir + "/cover.jpg")){
      rcover = dir + "/cover.jpg";
      cover = tags->setCover(w, h, rcover);
    }
    else if(QFile::exists(dir + "/cover.jpeg")){
      rcover = dir + "/cover.jpeg";
      cover = tags->setCover(w, h, rcover);
    }
    else if(NetCover && QFileInfo(dir).isWritable()){ //check if config allow cover download and if dir is writable
      if(tags->darkcover(dir)){
        rcover = dir + "/cover.jpg";
        cover = tags->setCover(w, h, rcover);
      }
    }
    if(cover == ""){
      rcover = "pics/nocover.png";
      cover = "pics/nocover.png";
    }
  }
}

void XMMSSensor::slotGetPos(){
  pos = xmms_remote_get_playlist_pos(0);
  if(pos != current){
    current = pos;
    emit songChange();
  }
}

void XMMSSensor::update(){
  QString format;
  QString imgsize;
  SensorParams *sp;
  Meter *meter;
  QObjectListIt it( *objList );
#ifdef HAVE_XMMS
  bool isRunning = xmms_remote_is_running(0);
  bool isPlaying = false;
  if( isRunning ){
    if( xmms_remote_is_main_win(0)){
      if( HideMainWin ){
        kdDebug("XMMS sensor") << "Hide XMMS main window" << endl;
        xmms_remote_main_win_toggle(0, false);
      }
    }
    else{
      if( !HideMainWin ){
        kdDebug("XMMS sensor") << "Show XMMS main window" << endl;
        xmms_remote_main_win_toggle(0, true);
      }
    }
    slotGetPos();
    isPlaying = xmms_remote_is_playing(0) || xmms_remote_is_paused(0);
    if(isPlaying){
      currentTime = xmms_remote_get_output_time( 0 );
    }
    else{
      currentTime = 0;
    }
  }
#endif // HAVE_XMMS
  while (it != 0){
    sp = (SensorParams*)(*it);
    meter = sp->getMeter();
#ifdef HAVE_XMMS
    if( isRunning ){
      format = sp->getParam("FORMAT");
      if (format.length() == 0 ){
          format = "%title %time / %length";
      }
      if( format == "%ms" ){
          meter->setMax( songLength );
          meter->setValue( currentTime );
      }
      else{
        if ( format == "%full" ){
            meter->setValue( 1 );
        }
        else{
          if(format.contains("%title")){
            format.replace( "%title", title );
          }
          if(EnableCover && format.contains("%cover")){
            format.replace( "%cover", cover );
          }
          if(EnableCover && format.contains("%viewer")){
            format.replace( "%viewer", Viewer );
          }
          if(EnableCover && format.contains("%rcover")){
            format.replace( "%rcover", rcover );
          }
          if(format.contains("%stitle")){
            format.replace( "%stitle", stitle );
          }
          if(format.contains("%artist")){
            format.replace( "%artist", artist );
          }
          if(format.contains("%album")){
            format.replace( "%album", album );
          }
          if(format.contains("%track")){
            format.replace( "%track", track );
          }
          if(format.contains("%rate")){
            format.replace( "%rate", QString::number(rate/1000) );
          }
          if(format.contains("%freq")){
            format.replace( "%freq", QString::number(freq) );
          }
          if(format.contains("%channel")){
          format.replace( "%channel", QString::number(channel) );
          }
          if(format.contains("%monoster")){
          format.replace( "%monoster", monoster );
          }
          if( format.contains("%runquit") ){
            format.replace("%runquit", "exit");
            meter->setValue( "exit" );
          }
          if(format == "%onoff"){
            format.replace( "%onoff", "xmms-shell -e xmmsexit" );
          }
          if(format == "%pref"){
            format.replace( "%pref", "xmms-shell -e preferences" );
          }
          if(format == "%play"){
            format.replace( "%play", "xmms-shell -e play" );
          }
          if(format == "%pause"){
            format.replace( "%pause", "xmms-shell -e pause" );
          }
          if(format == "%stop"){
            format.replace( "%stop", "xmms-shell -e stop" );
          }
          if(format == "%next"){
            format.replace( "%next", "xmms-shell -e next" );
          }
          if(format == "%prev"){
            format.replace( "%prev", "xmms-shell -e previous" );
          }
          if(format == "%volup"){
            format.replace( "%volup", "xmms-shell -e +" );
          }
          if(format == "%voldown"){
            format.replace( "%voldown", "xmms-shell -e -" );
          }
          if(format.contains("%vol")){
            vol = xmms_remote_get_main_volume( 0 );
            format.replace( "%vol", QString::number(vol) );
          }
          if(format.contains("%list")){
            if(xmms_remote_is_pl_win(0)){
              format.replace( "%list", "xmms-shell -e 'window playlist hide'" );
            }
            else{
              format.replace( "%list", "xmms-shell -e 'window playlist show'" );
            }
          }
          if(format.contains("%length")){
            format.replace( "%length", QTime( 0,0,0 ).addMSecs( songLength ).toString( "h:mm:ss" ) );
          }
          if(format.contains("%time")){
            format.replace( "%time", QTime( 0,0,0 ).addMSecs( currentTime ).toString( "h:mm:ss" ) );
          }
          if(isPlaying && format.contains("%remain")){
              format.replace( "%remain", QTime( 0,0,0 ).addMSecs( songLength ).addMSecs(-currentTime ).toString( "h:mm:ss" ) );
          }
          else if(format.contains("%remain")){
              format.replace( "%remain", QTime( 0,0,0 ).toString("h:mm:ss" ) );
          }
          meter->setValue(format);
        }
      }
    }
    else{
      format = sp->getParam("FORMAT");
      if(format == "%onoff"){
        format.replace( "%onoff", "xmms" );
        meter->setValue(format);
      }
      else if(format == "%title"){
        format.replace( "%title", "Karamba XMMS Sensor" );
        meter->setValue(format);
      }
      else if(format == "%cover" && EnableCover){
        format.replace( "%cover", "pics/nocover.png" );
        meter->setValue(format);
      }
      else if( format.contains("%runquit") ){
        format.replace("%runquit", "launch");
        meter->setValue(format);
      }
      else{
        meter->setValue("");
      }
    }
#endif // HAVE_XMMS
    ++it;
  }
  delete it;
  format = "";
}

void XMMSSensor::setMaxValue( SensorParams *sp){
    Meter *meter;
    meter = sp->getMeter();
    QString f;
    f = sp->getParam("FORMAT");

    if ( f == "%full" ){
        meter->setMax( 1 );
    }
}

void XMMSSensor::setKaramba(karamba* k)
{
	myKaramba = k;
}

void XMMSSensor::slotLoadConfig( ){
    KSimpleConfig conf("karambarc",true);
    conf.setGroup("karamba");
    Viewer =  conf.readEntry("ImgViewer", "kuickshow");
    conf.setGroup("XMMS");
    HideMainWin = conf.readBoolEntry("HideMainWin",false);
    EnableCover = conf.readBoolEntry("EnableCover",false);
    EnableID3 = conf.readBoolEntry("EnableId3",false);
    SaveID3 = conf.readBoolEntry("SaveId3",false);
    EnableCover = conf.readBoolEntry("EnableCover",false);
    EnableID3C = conf.readBoolEntry("EnableId3Cover",false);
    NetCover = conf.readBoolEntry("EnableNetCover",false);
    defH = conf.readNumEntry("CoverHeight",64);
    defW = conf.readNumEntry("CoverWidth",64);
}

